import asyncio
import os

import motor.motor_asyncio
import uvloop
from beanie import Document, init_beanie
from dotenv import load_dotenv
from pydantic import BaseModel

load_dotenv()

MONGO_PASSWORD = os.getenv("MongoDB_Password")
MONGO_IP = os.getenv("MongoDB_IP")
MONGO_USER = os.getenv("MongoDB_User")
MONGODB_PORT = os.getenv("MongoDB_Port")


class Events(Document):
    uuid: str
    date: str
    guild_id: int
    author_id: int
    month: int
    day: int
    year: int
    name: str
    description: str
    passed: bool


class EventsProjectDates(BaseModel):
    month: int
    day: int
    year: int
    passed: bool


class BerylEventsUtils:
    def __init__(self):
        self.self = self

    async def ins(
        self,
        uuid: int,
        date: int,
        guild_id: int,
        author_id: int,
        month_entry: int,
        day_entry: int,
        year_entry: int,
        name: str,
        description: str,
        passed: bool,
    ):
        """Inserts a new events entry into the database

        Args:
            uuid (int): The unique ID of the event
            date (int): The date of when it is added
            guild_id (int): Discord Guild ID
            author_id (int): Discord User ID
            month (int): Month of the event
            day (int): Day of the event
            name (str): Name of the event
            description (str): Description of the event
            passed (bool): Whether the event has already passed or not
        """
        client = motor.motor_asyncio.AsyncIOMotorClient(
            f"mongodb://{MONGO_USER}:{MONGO_PASSWORD}@{MONGO_IP}:{MONGODB_PORT}"
        )
        await init_beanie(database=client.beryl_events, document_models=[Events])
        entry = Events(
            uuid=uuid,
            date=date,
            guild_id=guild_id,
            author_id=author_id,
            month=month_entry,
            day=day_entry,
            year=year_entry,
            name=name,
            description=description,
            passed=passed,
        )
        await entry.create()

    asyncio.set_event_loop_policy(uvloop.EventLoopPolicy())

    async def listAll(self, user_id: int):
        """Returns all events in the database"""
        clientList = motor.motor_asyncio.AsyncIOMotorClient(
            f"mongodb://{MONGO_USER}:{MONGO_PASSWORD}@{MONGO_IP}:{MONGODB_PORT}"
        )
        await init_beanie(database=clientList.beryl_events, document_models=[Events])
        return await Events.find(Events.author_id == user_id).to_list()

    asyncio.set_event_loop_policy(uvloop.EventLoopPolicy())

    async def findEntry(self, user_id: int, name: str):
        """Finds 1 entry from a user's event based on the given name

        Args:
            user_id (int): User's Discord ID
            name (int): The name of the event
        """
        clientFindOne = motor.motor_asyncio.AsyncIOMotorClient(
            f"mongodb://{MONGO_USER}:{MONGO_PASSWORD}@{MONGO_IP}:{MONGODB_PORT}"
        )
        await init_beanie(database=clientFindOne.beryl_events, document_models=[Events])
        return await Events.find(
            Events.author_id == user_id, Events.name == name
        ).to_list()

    asyncio.set_event_loop_policy(uvloop.EventLoopPolicy())

    async def findEntryUpdate(
        self,
        user_id: int,
        original_name: str,
        new_name: str,
        description: str,
        month: int,
        day: int,
        year: int,
    ):
        """Finds 1 entry from a user's event based on the given name and updates the data

        Args:
            user_id (int): User's Discord ID
            name (int): The name of the event
        """
        clientFindOne = motor.motor_asyncio.AsyncIOMotorClient(
            f"mongodb://{MONGO_USER}:{MONGO_PASSWORD}@{MONGO_IP}:{MONGODB_PORT}"
        )
        await init_beanie(database=clientFindOne.beryl_events, document_models=[Events])
        mainUpdate = await Events.find_one(
            Events.author_id == user_id, Events.name == original_name
        )
        mainUpdate.name = new_name
        mainUpdate.description = description
        mainUpdate.month = month
        mainUpdate.day = day
        mainUpdate.year = year
        await mainUpdate.save()

    asyncio.set_event_loop_policy(uvloop.EventLoopPolicy())

    async def findProjection(self):
        """Finds projected values"""
        clientProjection = motor.motor_asyncio.AsyncIOMotorClient(
            f"mongodb://{MONGO_USER}:{MONGO_PASSWORD}@{MONGO_IP}:{MONGODB_PORT}"
        )
        await init_beanie(
            database=clientProjection.beryl_events, document_models=[Events]
        )
        return await Events.find_all().project(EventsProjectDates).to_list()

    asyncio.set_event_loop_policy(uvloop.EventLoopPolicy())

    async def setEventPassed(self):
        """Sets the event as passed"""
        clientPassed = motor.motor_asyncio.AsyncIOMotorClient(
            f"mongodb://{MONGO_USER}:{MONGO_PASSWORD}@{MONGO_IP}:{MONGODB_PORT}"
        )
        await init_beanie(database=clientPassed.beryl_events, document_models=[Events])
        mainPassed = Events.find(Events.passed == False)
        mainPassed.passed = True
        await mainPassed.save()

    asyncio.set_event_loop_policy(uvloop.EventLoopPolicy())

    async def removeEvent(self, uid: int, name: str):
        """Removes an event from the database

        Args:
            uid (int): Discord User's ID
            name (str): Name of the event
        """
        clientRmEvent = motor.motor_asyncio.AsyncIOMotorClient(
            f"mongodb://{MONGO_USER}:{MONGO_PASSWORD}@{MONGO_IP}:{MONGODB_PORT}"
        )
        await init_beanie(database=clientRmEvent.beryl_events, document_models=[Events])
        await Events.find(Events.author_id == uid, Events.name == name).delete()

    asyncio.set_event_loop_policy(uvloop.EventLoopPolicy())



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['beryl_events_utils']

package_data = \
{'': ['*']}

install_requires = \
['beanie>=1.11.2,<2.0.0',
 'python-dotenv>=0.20.0,<0.21.0',
 'uvloop>=0.16.0,<0.17.0']

setup_kwargs = {
    'name': 'beryl-events-utils',
    'version': '1.0.1',
    'description': "A set of async utils for Beryl's Events system",
    'long_description': "<div align=center>\n\n# Beryl-Events-Utils\n\n![PyPI](https://img.shields.io/pypi/v/beryl-events-utils?label=PyPi&logo=pypi&logoColor=white) ![PyPI - Python Version](https://img.shields.io/pypi/pyversions/beryl-events-utils?label=Python&logo=python&logoColor=white) ![PyPI - Downloads](https://img.shields.io/pypi/dd/beryl-events-utils?label=PyPi%20Downloads&logo=pypi&logoColor=white)\n\nA set of async utils for Beryl's Events system\n\n<div align=left>\n\n# Info\n\nBeryl, which is a discord bot offers a events system, where users can set an event and then later check how much more days is there left. The events system is powered off of MongoDB and thus requires an MongoDB server in order to use. These are the base corountines used by Beryl.\n\n# Installation\n\n## Pip\n\n```sh\npip install beryl-events-utils\n```\n\n## Poetry\n\n```sh\npoetry add beryl-events-utils\n```\n\n# License\n\nMIT\n",
    'author': 'No767',
    'author_email': '73260931+No767@users.noreply.github.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/No767/Beryl-Events-Utils',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
